package com.gic.auth.web.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.gic.auth.constant.MenuTypeEnum;
import com.gic.auth.web.vo.AppMenuVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.MenuLevelConstants;
import com.gic.auth.constant.MenuProjectConstants;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.dto.RoleDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.qo.RoleListQO;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.RoleApiService;
import com.gic.auth.service.UserApiService;
import com.gic.auth.web.vo.RoleDetailVO;
import com.gic.auth.web.vo.RoleSelectVO;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;

@RestController
@RequestMapping("/role")
public class RoleController {

    @Autowired
    private RoleApiService roleApiService;
    @Autowired
    private MenuApiService menuApiService;
    @Autowired
    private UserApiService userApiService;

    @RequestMapping("/get-detail")
    public RestResponse getDetail(Integer id) {
        return ResultControllerUtils.commonResultOne(roleApiService.getById(id), RoleDetailVO.class);
    }

    @RequestMapping("/save")
    public RestResponse save(RoleDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(roleApiService.save(dto));
    }

    @RequestMapping("/update")
    public RestResponse update(RoleDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(roleApiService.update(dto));
    }

    @RequestMapping("/delete")
    public RestResponse delete(Integer id) {
        return ResultControllerUtils.commonResult(roleApiService.delete(id));
    }

    @RequestMapping("/list-role")
    public RestResponse listRole(RoleListQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(roleApiService.pageRole(params));
    }

    @RequestMapping("/list-role-no-page")
    public RestResponse listRoleNoPage() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        return ResultControllerUtils.commonResult(roleApiService.listRole(enterpriseId), RoleSelectVO.class);
    }

    @RequestMapping("/list-menu")
    public RestResponse listMenu() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<UserDTO> userResponse = userApiService.getUserByEnterpriseId(enterpriseId);
        if (userResponse.isSuccess()) {
            UserDTO admin = userResponse.getResult();
            Integer userId = admin.getUserId();
            ServiceResponse<List<MenuDTO>> menuResponse = menuApiService.getSuperAdminMenu(userId);
            if (menuResponse.isSuccess()) {
                return RestResponse.success(menuResponse.getResult());
            } else {
                return EnterpriseRestResponse.failure(menuResponse);
            }
        } else {
            return EnterpriseRestResponse.failure(userResponse);
        }
    }

    /**
     * 查询列表数据
     * @Title: updateAppPage 
     * @Description:
      * @author guojuxing
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-gic-menu-tree")
    public RestResponse listMenuTree() {
        ServiceResponse<List<MenuDTO>> result = menuApiService.listMenuTreeForRole(MenuProjectConstants.DEFAULT_PROJECT);
        if (result.isSuccess()) {
            return RestResponse.success(changeListToTree(MenuLevelConstants.FIRST_LEVEL_PARENT_ID,  result.getResult()));
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("/list-app-menu-tree")
    public RestResponse listAppMenuTree() {
        return ResultControllerUtils.commonResult(menuApiService.listAppPageOfHasBuy(), AppMenuVO.class);
    }

    public static List<Map<String, Object>> changeListToTree(int storeGroupId, List<MenuDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (MenuDTO menuDTO : list) {
                if (storeGroupId == menuDTO.getParentId() && menuDTO.getIsShow().intValue() == 1) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("menuId", menuDTO.getMenuId());
                    data.put("menuName", menuDTO.getMenuName());
                    data.put("project", menuDTO.getProject());
                    data.put("menuType", menuDTO.getMenuType());
                    List<Map<String, Object>> children = changeListToTree(menuDTO.getMenuId(), list);
                    if (children == null || children.size() < 1) {
                        data.put("children", null);
                    } else {
                        data.put("children", children);
                    }
                    result.add(data);
                }
            }
        }
        return result;
    }
}
